#include <gtk/gtk.h>
#include <time.h>
#include <unistd.h>
#include "cat.h"

#include "scan.h"
#include "utils.h"

extern GdkPixmap *img_dir;
extern GdkBitmap *img_dir_mask;
extern GdkPixmap *img_dir_open;
extern GdkBitmap *img_dir_open_mask;
extern GdkPixmap *img_disk;
extern GdkBitmap *img_disk_mask;
extern GdkPixmap *img_disk_open;
extern GdkBitmap *img_disk_open_mask;

char *datetime(time_t time)
{
    char buf[128];

    strftime(buf, 128, "%d/%m/%Y %H:%M:%S", gmtime(&time));
    return g_strdup(buf);
}

char *readlink_alloc(const char *link)
{
    int size = 128;

    while (1) {
	char *buffer = g_new0(char, size);
	int nchars = readlink(link, buffer, size);
	if (nchars < size)
	    return buffer;
	g_free(buffer);
	size *= 2;
    }
}

static int nuke_entry(GNode *node, gpointer data)
{
    Item *item = ITEM(node->data);

    if (item) {
	if (item->name)
	    g_free(item->name);
	g_free(item);
    }
    return FALSE;
}

void nuke_disk(gpointer data, gpointer user_data)
{
    Disk *disk = DISK(data);
    g_node_traverse(disk->tree, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
	    nuke_entry, NULL);
    g_node_destroy(disk->tree);
    g_free(disk);
}

void nuke_catalog(Catalog *cat)
{
    g_list_foreach(cat->disks, nuke_disk, NULL);
    g_list_free(cat->disks);
    g_free(cat->catfile);
}

static int gnode_to_ctree(GtkCTree *ctree, unsigned int depth,
			  GNode *gnode, GtkCTreeNode *cnode, gpointer data)
{
    Item *item = ITEM(gnode->data);

    if (!cnode || !gnode)
	return FALSE;

    /* root node - make the disk label */
    if (!item) {
	Disk *disk = DISK(data);

	gtk_ctree_set_node_info(ctree, cnode, disk->label,
		UI_SPACING, img_disk, img_disk_mask, img_disk_open, img_disk_open_mask,
	    FALSE, FALSE);
	gtk_ctree_node_set_row_data(GTK_CTREE(ctree), cnode, disk);
	return TRUE;
    } else if (!item->is_leaf) {	/* this is a dir, so we create it */
	gtk_ctree_set_node_info(ctree, cnode, item->name,
	    UI_SPACING, img_dir, img_dir_mask, img_dir_open, img_dir_open_mask,
	    FALSE, FALSE);
	gtk_ctree_node_set_row_data(GTK_CTREE(ctree), cnode, gnode);

	/* let GTK update the UI */
	while (gtk_events_pending())
	    gtk_main_iteration();

	return TRUE;
    }

    /* it was a file and this ctree doesn't list files
     * this thaw seems to be needed in GTK? what the hell */
    gtk_clist_thaw(GTK_CLIST(ctree));
    return FALSE;
}

void fill_disk(gpointer data, gpointer user_data)
{
    Disk *disk = DISK(data);
    GtkCTree *ctree = GTK_CTREE(user_data);

    gtk_ctree_insert_gnode(ctree, NULL, NULL, disk->tree, gnode_to_ctree, disk);
}

void fill_disks(GtkCTree *ctree, Catalog *cat)
{
    /* empty catalog, dont bother */
    if (cat->disks == NULL)
	return;
    
    gtk_clist_clear(GTK_CLIST(ctree));
    gtk_clist_freeze(GTK_CLIST(ctree));
    g_list_foreach(cat->disks, fill_disk, ctree);
    gtk_clist_thaw(GTK_CLIST(ctree));
}

void cat_add_disk(Ui *ui, const char *label, const char *path)
{
    Catalog *cat = ui->cat;
    Disk *disk = NULL;

    disk = g_new0(Disk, 1);
    cat->disks = g_list_append(cat->disks, disk);
    g_snprintf(disk->label, 20, "%s", label);
    disk->tree = g_node_new(NULL);

    update_statusbar(ui, SB_SCANNING);
    scan_tree(disk->tree, path, TRUE);
    update_statusbar(ui, SB_UPDATING);

    gtk_ctree_insert_gnode(GTK_CTREE(ui->dirtree), 
		NULL, NULL, disk->tree, gnode_to_ctree, disk);

    update_statusbar(ui, SB_READY);
}

void update_statusbar(Ui *ui, int mode)
{
    const char *message = NULL;

    switch (mode) {
	case SB_READY:
	    message = "Ready";
	    break;
	case SB_UPDATING:
	    message = "Updating...";
	    break;
	case SB_SCANNING:
	    message = "Scanning...";
	    break;
	case SB_SAVING:
	    message = "Saving...";
	    break;
	case SB_EJECTING:
	    message = "Ejecting...";
	    break;
	case SB_SEARCHING:
	    message = "Searching...";
	    break;
	default:
	    message = "";
	    break;
    }

    gtk_statusbar_pop(GTK_STATUSBAR(ui->status), 1);
    gtk_statusbar_push(GTK_STATUSBAR(ui->status), 1, message);
}
